<title>PHP Engines Maze Generator</title>
PHP Engines Maze Generator example
<style type="text/css">
.container {
	display: table;
	table-layout: fixed;
	border: 5px solid black;
	border-collapse: collapse;
}

.row { display: table-row; }

.cell {
	display: table-cell;
	width: 40px;
	height: 40px;
}

.start { background-color: #22FF22; }
.end { background-color: #FF2222; }

.n { border-top: 3px solid black; }
.s { border-bottom: 3px solid black; }
.e { border-right: 3px solid black; }
.w { border-left: 3px solid black; }

</style>
<?php

class Maze
{
	private $maze;				//Maze stored as multi-dimensional array
	
	private $start_point;		//Start point object
	private $end_points;    	//Array of end point objects
	private $point_spacing;		//Minimum distance start/end points must be from each other
	
	private $maze_height;		//Width of maze in cells
	private $maze_width;		//Height of maze in cells
	
	
	function __construct($end_points = 1, $width = 20, $height = 10, $spacing = 2)
	{
		//Set height + width + point spacing from default or input data
		$this->maze_width = $width;
		$this->maze_height = $height;
		$this->point_spacing = $spacing;
		
		//Init array with default data (not visited, not a start or end point, all walls closed
		for ($i = 1; $i <= $this->maze_height; $i++)
		{
			for ($j = 1; $j <= $this->maze_width; $j++)
			{
				$this->maze[$i][$j] = '0001111'; //Key: visited, start, end, N, S, E, W
			}
		}
		
		//Create start point
		$this->start_point = $this->get_rand_point();
		$this->maze[$this->start_point->y][$this->start_point->x]{1} = '1';
		
		//Traverse maze to knock down walls
		$this->traverse_maze();
		
		//Create end points, after knocking down walls to ensure good exit placement
		$result = $this->create_end_points($end_points);
		//if (!$result)
			//echo "Could not create the necessary number of end points with desired spacing.";
	}

	
	function print_maze()
	{
		echo "<div class=\"container\">\n";
		for ($i = 1; $i <= $this->maze_height; $i++)
		{
			echo "\t<div class=\"row\">\n";
			for ($j = 1; $j <= $this->maze_width; $j++)
			{
				$cell_value = $this->maze[$i][$j];
				$class_str = $this->get_cell_classes($cell_value);
				
				echo "\t\t<div class=\"$class_str\"></div>\n";
			}
			echo "\t</div>";
		}
		
		echo '</div>';
	}
	
	
	private function traverse_maze()
	{
		//Init values for traversal. Only starting point visited.
		$moves = array();
		$cell_count = $this->maze_height * $this->maze_width;

		$curr_point = clone $this->start_point;
		$this->maze[$curr_point->y][$curr_point->x]{0} = 1;
		$visited_points = 1;

		//Continue traversal until all points visited in some manner
		while ($visited_points < $cell_count)
		{
			$possible_moves = '';
	
			//Check neighbors of current point to find directions with an unvisited point still within maze bounds
			if ($curr_point->y > 1 && $this->maze[$curr_point->y - 1][$curr_point->x]{0} == 0)
				$possible_moves .= 'N';
			if ($curr_point->y < $this->maze_height && $this->maze[$curr_point->y + 1][$curr_point->x]{0} == 0)
				$possible_moves .= 'S';
			if ($curr_point->x < $this->maze_width && $this->maze[$curr_point->y][$curr_point->x + 1]{0} == 0)
				$possible_moves .= 'E';
			if ($curr_point->x > 1 && $this->maze[$curr_point->y][$curr_point->x - 1]{0} == 0)
				$possible_moves .= 'W';

			if ($possible_moves)
			{
				//Visiting new point, increment visited counter
				$visited_points++;
				
				//Add current move to moves array for backtracking later, clone needed as objects are normally passed by reference
				$temp_point = clone $curr_point;
				$moves[] = $temp_point;
				
				//Randomly determine next direction to go from available moves
				$direction = $possible_moves{rand(0, strlen($possible_moves) - 1)};

				//Based on direction chosen, knock down walls and re-assign current point value
				switch($direction){
					case "N":
						$this->maze[$curr_point->y][$curr_point->x]{3} = 0;
						$this->maze[$curr_point->y - 1][$curr_point->x]{4} = 0;
						
						$curr_point->y--;
						break;
					case "S":
						$this->maze[$curr_point->y][$curr_point->x]{4} = 0;
						$this->maze[$curr_point->y + 1][$curr_point->x]{3} = 0;
						
						$curr_point->y++;
						break;
					case "E":
						$this->maze[$curr_point->y][$curr_point->x]{5} = 0;
						$this->maze[$curr_point->y][$curr_point->x + 1]{6} = 0;
						
						$curr_point->x++;
						break;
					case "W":
						$this->maze[$curr_point->y][$curr_point->x]{6} = 0;
						$this->maze[$curr_point->y][$curr_point->x - 1]{5} = 0;
						
						$curr_point->x--;
						break;
				}

				//Set point as visited
				$this->maze[$curr_point->y][$curr_point->x]{0} = 1;
			}
			else
			{
				//No moves possible, backtrack
				$curr_point = array_pop($moves);
			}
		}
	}
	
	
	private function create_end_points($end_points)
	{		
		//Array will hold all possible good end points
		$potential_points = array();
		$points = 0;
		
		//Scan array for potential end points
		for ($i = 1; $i <= $this->maze_height; $i++)
		{
			for ($j = 1; $j <= $this->maze_width; $j++)
			{
				$walls = substr($this->maze[$i][$j], 3);
				$num_walls = substr_count($walls, '1');
				
				//Only want 3 walled points. This ensure exits are always at the end of a path
				if ($num_walls == 3)
					$potential_points[] = new Point($j, $i);
			}
		}
		
		//Attempt to create as many end points as needed, minding spacing needs
		while ($points < $end_points)
		{
			//Check that we have points to work with. If not, we cannot create end points, error out
			if (empty($potential_points))
				return FALSE;
				
		
			//Shuffle potential points, pop off last one for examination
			shuffle($potential_points);
			$new_point = array_pop($potential_points);
			
			//Check if far enough from start point
			if ((abs($new_point->x - $this->start_point->x) < $this->point_spacing) || (abs($new_point->x - $this->start_point->y) < $this->point_spacing))
				continue;
				
			//Check if far enough from other end points
			if (is_array($this->end_points))
			{
				foreach ($this->end_points AS $end_point)
				{
					if ((abs($new_point->x - $end_point->x) < $this->point_spacing) || (abs($new_point->x - $end_point->y) < $this->point_spacing))
						continue 2;
				}
			}
			
			//Add to end points arrays, adjust maze
			$this->end_points[] = $new_point;
			$this->maze[$new_point->y][$new_point->x]{2} = '1';
			
			//Valid point, increment point counter
			$points++;
		}
		
		return TRUE;
	}
	
	
	private function get_rand_point()
	{
		//Get random coordinates within maze bounds
		$point_x = rand(1, $this->maze_width);
		$point_y = rand(1, $this->maze_height);
		
		return new Point($point_x, $point_y);
	}
	
	
	private function get_cell_classes($string)
	{
		//Init blank array
		$classes = array('cell');
		
		//Check if CSS class should be added for cell
		if ($string{1}) $classes[] = 'start';
		if ($string{2}) $classes[] = 'end';
		if ($string{3}) $classes[] = 'n';
		if ($string{4}) $classes[] = 's';
		if ($string{5}) $classes[] = 'e';
		if ($string{6}) $classes[] = 'w';
		
		//Join all valid array pieces in one string for CSS class value
		$class_str = implode(' ', $classes);
		
		return $class_str;
	}
	
}

class Point
{
	public $x;
	public $y;
	
	function __construct($x = 0, $y = 0)
	{
		$this->x = $x;
		$this->y = $y;
	}
	
	function get_coords()
	{
		return "($this->x, $this->y)";
	}
}

$maze = new Maze(5, 20, 10, 4);
$maze->print_maze();
?>